/****************************************************************************
**
** Copyright (C) 2003-2007 Trolltech ASA. All rights reserved.
**
** This file is part of a Qt Solutions component.
**
** This file may be used under the terms of the GNU General Public
** License version 2.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.  Please review the following information to ensure GNU
** General Public Licensing requirements will be met:
** http://www.trolltech.com/products/qt/opensource.html
**
** If you are unsure which license is appropriate for your use, please
** review the following information:
** http://www.trolltech.com/products/qt/licensing.html or contact the
** Trolltech sales department at sales@trolltech.com.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/
#include <QtGui>
#include <qtbrowserplugin.h>

#ifdef QAXSERVER
#include <ActiveQt/QAxBindable>
#endif

#include <plib/ssg.h>
#include "glcontrol.hpp"
#include "messages.hpp"
#include "astar.hpp"
#include "qcakerun.h"

#include "../quazip/quazip.h"
#include "../quazip/quazipfile.h"
#include "../quazip/gojobzip.h"

// global variables !!!!
int g_mouse_mode=0;
int g_mouse_x=0;
int g_mouse_y=0;
int g_mouse_button=0;
sgVec3 g_poi; // point of interest
int g_window_w=0;
int g_window_h=0;
QSet<int> g_keyset;
int g_key=0;
int g_playmode=0;
int g_switch = 0;
long g_nextid=0;

Glcontrol    *g_gl=NULL;
QString g_prjpath;
QString g_apppath;
QString g_editor;
// Catalog
QString g_3dcatalogpath;
QString g_scriptcatalogpath;
QString g_soundcatalogpath;
QString g_shadercatalogpath;
QString g_physikcatalogpath;
// Display
int g_displayfps;
int g_displayphfps;

Messages *g_messages = NULL;

FolderListItem *g_tmpItem=NULL;
FolderListItem *g_switchScene = NULL;
// AStar
aStar *gastar= NULL;

class QCweb3d : public QWidget, public QtNPBindable
#ifdef QAXSERVER
            , public QAxBindable
#endif
{
    Q_OBJECT
    Q_PROPERTY(QString text READ text WRITE setText)

    Q_CLASSINFO("ClassID", "{4a22176d-118f-4185-9653-9f98958a6df8}")
    Q_CLASSINFO("InterfaceID", "{1df735ba-da4f-4fb7-8f35-b8dfbf8cfd9a}")
    Q_CLASSINFO("EventsID", "{349de213-f8bd-4d2e-a2cf-eab407c03245}")

    Q_CLASSINFO("MIME", "qcweb3d/very:xxx:QCweb3d and useless")
//    Q_CLASSINFO("MIME", "qcweb3d/very:xxx:QCweb3d and useless")
//    Q_CLASSINFO("MIME", "application/x-graphable:g1n:Graphable ASCII numeric data")

    Q_CLASSINFO("ToSuperClass", "QCweb3d")
    Q_CLASSINFO("DefaultProperty", "text")

public:
    QCweb3d(QWidget *parent = 0);
    ~QCweb3d();

    void mousePressEvent(QMouseEvent *);
    void mouseMoveEvent(QMouseEvent *e);
    void mouseReleaseEvent(QMouseEvent*);
    QString text() const;
    bool readData(QIODevice *source, const QString &format);
    void transferComplete(const QString &url, int id, Reason r);


public slots:
    void about();
    void setText(const QString &text);

signals:
    void mouseDown();
    void mouseMove(int x, int y);
    void mouseUp();

protected:
    void paintEvent(QPaintEvent*);

private:
    void processData(QDataStream &in);

    QString m_text;
    QcakeRunApp  *gl_run;
    int lastReqId, lastConfId;
    QString lastConfUrl;
    Reason lastConfReason;
};


QCweb3d::QCweb3d(QWidget *parent)
: QWidget(parent)
{
    m_text = QString::fromLatin1("Empty");
    g_gl = new Glcontrol(this);
    gl_run = new QcakeRunApp(this);
    QHBoxLayout *layout = new QHBoxLayout;
    layout->addWidget(g_gl);
    this->setLayout(layout);
    this->show();

    lastReqId = 0;
    lastConfId = 0;
    lastConfReason = ReasonUnknown;
}

QCweb3d::~QCweb3d()
{
  if(gastar)
      delete gastar;
  qDebug("destroy QCweb3d");
}

void QCweb3d::about()
{
     QMessageBox::aboutQt(this);
}

void QCweb3d::setText(const QString &text)
{
     m_text = "/tmp/qcwebgame/" + text;
     update();
}

void QCweb3d::mousePressEvent(QMouseEvent *)
{
    mouseDown();
}

void QCweb3d::mouseMoveEvent(QMouseEvent *e)
{
    mouseMove(e->x(), e->y());
}

void QCweb3d::mouseReleaseEvent(QMouseEvent*)
{
    mouseUp();
}

QString QCweb3d::text() const
{
    return m_text;
}

void QCweb3d::paintEvent(QPaintEvent*)
{
    QPainter p(this);
    QRect r(rect());
//        g_gl->show();
//        g_gl->resizeGL( 600, 400 );
//          resize(300,300);
//        g_gl->resizeGL( r.width(), r.height() );
//        r.adjust(0, 0, -1, -1);
//        p.drawRect(r);
//        p.drawText(r, Qt::AlignCenter, m_text);
}

bool QCweb3d::readData(QIODevice *source, const QString &/*format*/)
{
    qDebug("QCweb3d::readData");
    if (!source->open(QIODevice::ReadOnly))
        return false;

    QDataStream in(source);
    processData(in);
    qDebug() << "Text" + m_text;
    if(g_gl){
      gl_run->openDocumentFile(m_text);
      gl_run->slotToolsPlay();
    }
    update();
    return true;
}

void QCweb3d::processData(QDataStream &in)
{
  char mybuff[256];
  int  mylen=0;
  QTextStream msgout(stdout);

  // load data to tmp
  QFile file("/tmp/qcweb3dfile.zip");
  file.open(QIODevice::WriteOnly);
  QDataStream out(&file);   // we will serialize the data into the file
  while (!in.atEnd()) {
//       out << in;   // serialize
         mylen=in.readRawData ( mybuff, 255 );
         out.writeRawData ( mybuff, mylen );
  }
  file.close();

// unzip data
  QString a2 , a3, la, dis, folderzip;
  a2="/tmp/qcweb3dfile.zip";
  a3="/tmp/qcwebgame";
  out << "Start unzip file... " << a2 << " \n";
  QDir check(a3);

  check.mkpath(a3);
  if (check.exists() && is_file(a2)) {
      dis = check.absolutePath();
      if (!dis.endsWith("/")) {
           dis.append("/");
      }
      bool uz = UnzipTo(a2,dis);
      if (uz) {
        msgout << "UnZip " << a2 << " is ready.\n"; 
      } else {
         msgout << "UnZip " << a2 << " file or dir not found.\n";   
      }
      
  } else {
      msgout << "UnZip " << a2 << " file or dir not found.\n"; 
  }
}

void QCweb3d::transferComplete(const QString &url, int id, Reason r)
{
    lastConfId = id;
    lastConfUrl = url;
    lastConfReason = r;
}

#include "qcweb3d.moc"

QTNPFACTORY_BEGIN("QCweb3d Plugin", "A QCake-based LiveConnected plug-in that plays qcake files")
    QTNPCLASS(QCweb3d)
QTNPFACTORY_END()

#ifdef QAXSERVER
#include <ActiveQt/QAxFactory>
QAXFACTORY_BEGIN("{9a3216bf-7e20-482c-84c6-06167bacb616}", "{07538ca5-eb7a-4f24-a3c4-a120c6e04dc4}")
    QAXCLASS(QCweb3d)
QAXFACTORY_END()
#endif
